require 'raci'
require 'csv'

class Hermes::Admin::UntensedAreasController < Hermes::Admin::BaseController
	before_action :set_area, only: %i[destroy update]

	def index
		@areas = Hermes::UntensedArea.readable_by(current_user).paginate(page: params[:page], per_page: 20)
	end 

	def create
		file = params[:csv]
		unless file.nil?
			tfile = file.to_io

			file_r = tfile.read
			CSV.parse(file_r, headers: true, encoding: 'iso-8859-1:UTF-8') do |row|
				department = row["departement"]
				insee_code = row["code_insee"]
				if department.present? && insee_code.present?
					area = Hermes::UntensedArea.find_or_create_by(department: department, insee_code: insee_code)
					area.set(user_id: current_user.id, updated_at: DateTime.current)
				end
			end 
		end 
		redirect_to_root
	end

	def update
		if @area.update(area_params)
			flash[:notice] = 'Zone modifié'
		else
			flash[:alert] = @area.errors.full_messages
		end

		redirect_to_root
	end

	def download_csv 
			send_file "#{Rails.root}/packages/hermes/app/assets/docs/area_exemple.csv", type: "application/csv", x_sendfile: true
	end

	def destroy
		if @area.destroy
			flash[:notice] = 'Zone supprimé'
		else
			flash[:alert] = @area.errors.full_messages
		end

		redirect_to_root
	end

	private

	def set_area
		@area = Hermes::UntensedArea.find params[:id]
	end

	def area_params
		params.require(:untensed_area).permit(
			:insee_code,
			:department
		)
	end

	def redirect_to_root
		redirect_to admin_untensed_areas_path(page: params[:page])
	end
end 
