require 'vcr_config' if Rails.env == 'test'
require "base64"

module Hermes
  module Kyc
    # Handlers for transition from old to enw
    class KycBase
      
      def start_treatment (doc_to_kyc, k7_name = nil)
        @required_document = doc_to_kyc
        kyc_response = get_kyc_response(k7_name)
        return unless kyc_response
        
        analysis_response(kyc_response)
        
        @required_document.status = @analysed_response[:status]
        @required_document.validation_errors = @analysed_response[:validation_errors]
        @required_document.validity_limit = @analysed_response[:validity_limit]
        @required_document.metas = @analysed_response[:metas]
        @required_document.save
      end
      def try_call_netheos (doc_to_kyc, k7_name = nil)
        @required_document = doc_to_kyc
        kyc_response = get_kyc_response(k7_name)
  return JSON.parse kyc_response.raw_body
      end
      def get_treatment_holder_params
        applicant = @required_document.applicant
        holder = {firstName: applicant.first_name,
                  lastName: applicant.last_name,
                  birthDate: applicant.birth_date&.to_date
        }
        holder[:gender] = applicant.civility == "1" ? "M": "F"
        holder[:birthName] = applicant.patronymic_name || applicant.last_name
        holder.to_json
      end
  
      def get_kyc_response(k7_name = nil)
        holder_params = get_treatment_holder_params
        path = get_route
        if (Rails.env == 'test')
          VCR.use_cassette("send_file_to_metheos" + (k7_name ? k7_name.to_s : ""), :match_requests_on => [:method, :uri, :path]) do
            call_kyc(path, holder_params)
          end
        else
          call_kyc(path, holder_params)
        end
      end
      def holder_field
        "holder"
      end
      def get_tax_field
        
      end
      def add_verso
        return false
      end
      def call_kyc(path, holder_params)
        request = Curl::Easy.new(HERMES_CONFIG['kyc']['url'] + path)
        request.proxy_url = APP_CONFIG['proxy'] || ENV['http_proxy']
        request.headers["Accept"] = "application/json"
        doc_to_post = Doc.find_by(id: @required_document.doc_id)
        return nil unless doc_to_post && doc_to_post.data
        
        request.multipart_form_post = true
        request.http_auth_types = :basic
        request.username = HERMES_CONFIG['kyc']['login']
        request.password = HERMES_CONFIG['kyc']['password']
        
        post_field = Curl::PostField.content('file_1', doc_to_post.data)
        post_field.remote_file = doc_to_post.title.cleaned
        post_field.content_type = doc_to_post.content_type
        extra_args = [
          post_field,
          Curl::PostField.content(holder_field, holder_params),
        ]
        if add_verso
          post_field2 = Curl::PostField.content('file_2', doc_to_post.data)
          post_field2.remote_file = doc_to_post.title.cleaned
          post_field2.content_type = doc_to_post.content_type
          extra_args.push(post_field2)
        end
        if get_tax_field
          extra_args.push(get_tax_field)
        end
        request.ssl_verify_peer = false
        request.http_post(extra_args)
        resp = WsResponse.from_curl(request)
        resp
      end
      
      def check_fields(simple_hash)
        simple_hash.keys.each do |key|
          if simple_hash[key].is_a? String
            if simple_hash[key] == "KO"
              @analysed_response[:status] = 'Pièce invalide'
              @analysed_response[:validation_errors].push({message: key , status: "KO"})
            end
          elsif simple_hash[key].is_a? Hash
            check_fields(simple_hash[key])
          end
        end
      end
      
      def check_or_fields(simple_hash)
        return unless simple_hash
        counter = simple_hash.keys.count
        simple_hash.keys.each do |key|
          if (simple_hash[key].is_a? String) && simple_hash[key] == "KO"
              counter = counter - 1
            end
        
        end
        if counter <= 0
          check_fields(simple_hash)
        end
      end
      
      def common_check(parsed_body)
        check_fields(parsed_body["controlsGroups"]["document"].slice(
          "typeRecognized",
         # "quality",
         # "aboveMinimumThreshold",
         # "noAnnotationFound",
         # "documentAspect",
        #  "securityElements"
        ))
      
      end
      
      def specific_check(parsed_body)
        controls_groups_check(parsed_body)
        document_check(parsed_body)
        custom_check(parsed_body)
      end
      def add_aditionnal_metas(parsed_body)
        
      end
      def analysis_response(kyc_response)
        @analysed_response = {
          status: 'Pièce validée',
          validation_errors: [],
          validity_limit: nil,
          metas: {}
        }
        begin
          parsed_body = JSON.parse kyc_response.raw_body
        rescue JSON::ParserError
          @analysed_response[:status] = 'Pièce invalide'
          @analysed_response[:validation_errors] = [
            {message: "processus_error",
             status: "KO",
             status_code: 500,
             date_time: DateTime.current,
             error_message: kyc_response.raw_body}]
        end
       
   
        if (kyc_response.status_code == 200)
          
          common_check(parsed_body)
          specific_check(parsed_body)
          @analysed_response[:metas] = {
            "documentType" => parsed_body["documentType"],
            "documentDetails" => parsed_body["documentDetails"],
            "controlsGroups" => parsed_body["controlsGroups"],
            
          }
          add_aditionnal_metas(parsed_body)
        elsif (kyc_response.status_code == 400)
          @analysed_response[:status] = 'Pièce invalide'
          @analysed_response[:validation_errors] = [{
                                                      message: parsed_body["error"]["message"], 
                                                      status: "KO", status_code: 400}
          ]
        else
          @analysed_response[:status] = 'Pièce invalide'
          @analysed_response[:validation_errors] = [
            {message: "processus_error",
             status: "KO",
             status_code: kyc_response.status_code,
             date_time: DateTime.current,
             error_message: parsed_body["error"]["message"]}]
        end
      
      
      end
    end
  end
end